# WidgetBuilder

By: Endri Gjiri *www.name-reaction.com*

**WidgetBuilder** aims to be a simple way to create javascript widgets. It allows developers to work separately on the html, css, and javascript files for their widgets. WidgetBuilder combines all of these files into a single minified javascript file, ready to be loaded by the end user. WidgetBuilder also wraps everything with a javascript Widget class, namespacing everything, and allows the display of multiple widgets in the same page without conflicts.
#### Other Features include:
- Provides a simple and small snippet for the user on which to load the widget. It only consists of an anchor tag with a class of the specific ```widget_name``` and the script tag to load the generated widget javascript.
- The a tag on which the widget is loaded on top of can serve as a way to pass options to the widget js through the a tag's data attributes. This data can be used to customize the widget's function or style.
- Provides a ```validateReference``` function that checks if the developer's site is referenced properly SEO-wise and if not it destroys the widget. This function needs to be called explicitly in the developer's custom javascript.
- Provides a ```resetCSS``` function which minimizes style conflicts with the sites that the widget ultimately ends up in.

## Installation

Add this line to your application's Gemfile:
```ruby
gem 'widget_builder'
```

And then execute:
```console
$ bundle install
```

Or install it yourself as:
```console
$ gem install widget_builder
```

### Brief description of the Ruby classes

WidgetBuilder consists of a Ruby module of the same name which contains two classes named **Widget** and **Widgets** (plural).

The main class is the **Widget** class which needs 3 arguments, the html_path, css_path, and js_path respectively. It also has a forth optional argument which is an options hash. This class essentially reads all of these files, includes the css inline into the html, and then adds this combined html_and_css string into the javascript. This class also wraps the javascript with the gem's own ```widget_loader``` javascript which provides additional functionality to the widget. Instance methods include: ```save```, ```show_snippet```, ```report```, and ```compile```, among others.

The **Widgets** (plural) class is designed to create multiple widgets at once, relying on naming conventions. It also accepts html_path, css_path, and js_path as arguments but in this case they do not refer to the file paths but instead to directory paths. This class will first go to the html directory and get all the file names and try to match them to css and js files of the same name in their respective folders. It will then loop through the list of html files and create an instance of the Widget class by passing in the html, css, and js file. The final result is an array of widgets (instances of Widget). The Widgets class also responds to ```save```, ```report```, and ```compile``` and loops over the array of widgets and calls the appropriate method on the widget instance.

## Usage

#### The simplest way to get started with this gem is the following:

1. Place the html, css, and js files needed for the widget all in the same folder and navigate there in the console.

2. Load irb and require widget_builder
```console
$ irb
> require 'widget_builder'
```

3. Compile the widget directly through irb.
```ruby
WidgetBuilder::Widgets.new.compile
```
Note: When no paths are passed as attributes to the Widgets class, it defaults them to the current directory.

This will create a new javascript file which is the new widget file, named according to the widget_name. The widget_name gets extracted by the class of the first div in the html or it can be passed in through the options hash.

This command will also generate a report in the console, also including an html snippet that the end user can copy and place on their pages to display the widget.

A sample snippet is shown below:
```html
<a class="chemiphi_MMC" href="http://www.name-reaction.com"></a>
<script type="text/javascript">
   var element = document.createElement('script');
   element.type = 'text/javascript';
   element.src = 'http://api.chemiphi.com/public_widgets/chemiphi_MMC.js';
   document.getElementsByTagName('head')[0].appendChild(element);
</script>
```

## Usage with Rails
#### Building a widget by specifying the paths to the html, css, and js in a Rails app.

A simple way to build a widget in rails is to do it by creating a custom Task.

Let's create a new Ruby file "/lib/tasks/widget_compiler.rake" with the following code:

```ruby
desc "Creates a single uglified widget JavaScript file containing the custom HTML, CSS, and JS"
task :create_widget do
    html_path = File.join(Rails.root, 'app', 'views', 'widgets', 'my_widget.html')
	css_path = File.join(Rails.root, 'app', 'assets', 'stylesheets', 'widgets', 'my_widget.css')
	js_path = File.join(Rails.root, 'app', 'assets', 'javascripts', 'widgets', 'my_widget.js')

	widget = WidgetBuilder::Widget.new(html_path, css_path, js_path)

    export_path = File.join(Rails.root, 'public', 'widgets')
	widget.compile(export_path)
end
```
In the above code we have specified that the final widget js should be placed in a public/widgets folder so before we run this task we need to first create the widgets folder.

We can then build the widget by running the following command from the console:
```console
$ rake create_widget
```
This will create the widget.js in the public/widgets folder, ready to be accessed by users and also displays in the console the HTML snippet required to display this widget.

## Options

#### The WidgetBuilder also accepts an options hash as a fourth argument.

Available options include:
- **compress** - *defaults to true* - specifies whether the compiled javascript file should be minified.
- **href** - *defaults to #* - specifies the url that the snippet anchor tag should link to. This has to be passed in as an option if the *validateReference* function is called from the js file.
- **source_path** - needed to point to the src of the compiled widget.js in the snippet. In the above example this could be *www.my_domain.com/widgets*.

**Example:**
```ruby
options = { compress: false, href: 'www.my_domain.com', source_path: 'www.my_domain.com/widgets' }
widget = WidgetBuilder::Widget.new(html_path, css_path, js_path, options)
widget.compile(export_path)
```

This would create a non-minified widget javascript and final HTML snippet that looks like this:
```html
<a class="my_widget" href="http://www.my_domain.com"></a>
<script type="text/javascript">
   var element = document.createElement('script');
   element.type = 'text/javascript';
   element.src = 'http://www.my_domain.com/widgets/my_widget.js';
   document.getElementsByTagName('head')[0].appendChild(element);
</script>
```
## JavaScript Utilities
#### The widget's functionality is determined in the custom javascript file that you develop.
##### This javascript get wrapped in the gem's ```widget_loader``` js code that extends it with additional functionality.

- ```var widget ``` becomes available to your custom javascript file and it stores the whole widget object. Outputting it to the console shows that it includes the widget ```id```, ```aTag```, ```dataset```, ```app```, and the ```validateRefence``` function.
- ```var w ``` is also available to the custom javascript and it holds the whole widget as a jQuery object. This can come in handy when trying to find elements within a particular widget and can be done using ```w.find('.my_element') ``` instead of searching it on the global scope as in ```$(.my_element) ```. This can be useful when it is desirable to have multiple widgets running on the same page without conflicting with each other.

## Future Work
- As of now the only file formats that are supported are html, css, js as well as scss, and sass. In the next release I would like to extend support for haml and coffeeScript. 
- Current integration with Rails is quite raw and does not take advantage of the capabilities of Rails. In the future I want to create another gem, likely wiget_builder-rails, that is designed specifically for working in Rails.

## Contributing

1. Fork it
2. Create your feature branch (`git checkout -b my-new-feature`)
3. Commit your changes (`git commit -am 'Add some feature'`)
4. Push to the branch (`git push origin my-new-feature`)
5. Create new Pull Request
